import urllib2, urllib
import sys, os
import logging

class HttpRequestor(object):
	"""
	Helper class for sending requests of HTTP with GET and POST
	"""

	apiUrl = "" 				#: domain url to use when sending requests
	apiMethod = "" 			#: api method to use when sending requests
	apiParams = "" 			#: parameters to send to the method when sending requests
	apiHeader = "" 			#: Authorization to put into header when sending requests
	logger = None 				#: logger to send loggin information to.  Logger comes from pytest test definitions
	apiPathParams = ""
	formattedParams = ""

	def __init__(self,apiUrl,logger):

		self.apiUrl = apiUrl
		self.logger = logger

	def makePathParams(self,params):
		"""
		Constucts a request with the parameters formated in path parameter format. 
			EX: http://myserver.com/api/method/pathParam1/pathParam2
		
		Args :null

		Returns :null

		Raises :null

		"""
	
		pathParams = ""

		for param in params:
			pathParams = pathParams+"/"+param

		#path params must be at the start of the parameter list
		self.formattedParams = urllib.quote_plus(pathParams,"/")+self.formattedParams
		
	def makeQueryParams(self,params):
		"""
		Constucts a request with the parameters formated in query parameter format. 
			EX: http://myserver.com/api/method?queryParam=paramValue
		
		Args :null

		Returns :null

		Raises :null

		"""
	
		queryParams = ""

		encoded_param = urllib.urlencode(params)

		#query params must be at the end of the paramater list
		self.formattedParams = self.formattedParams+"?"+encoded_param



	def makeGetRequest(self):
		"""
		Sends a GET HTTP request.  Make sure you have set the classe's apiParams,apiUrl and apiMethod first

		Args :null

		Returns : (file-like object) the HTTP response 

		Raises :null
		"""
		
		url = self.apiUrl+self.apiMethod
		request = urllib2.Request(url+self.formattedParams)
		self.logger.info(request.get_full_url())
		self.logger.info(request.get_data())
		self.logger.info(request.get_method())

		try:

			return urllib2.urlopen(request)

		except urllib2.URLError,e:
			return e
		finally:
			self.formattedParams = "" 


	def makePostRequest(self):
		"""
		Sends a POST HTTP request.  Make sure you have set the classe's apiParams,apiUrl, apiHeader and apiMethod first

		Args :null

		Returns : (file-like object) the HTTP response 

		Raises :null
		"""
		url = self.apiUrl+self.apiMethod
		request = urllib2.Request(url)
		
		encoded_param = urllib.urlencode(self.apiParams)
		request.add_data(encoded_param)
		request.add_header("Authorization",self.apiHeader)

		try:

			self.logger.info(request.get_full_url())
			self.logger.info(request.get_data())
			self.logger.info(request.get_method())
			return urllib2.urlopen(request)


		except urllib2.URLError,e:
			return e
		finally:
			self.formattedParams = "" 


	def getAuth(self,credentials):
		"""
		Creates the authentication for accessing HTTP

		Args :
		credentials : (dict) 'username' and 'password' keys for a dictionary containing the HTTP Basic Auth to access the api

		Returns : null

		Raises :null
		"""

		# create a handler for the username / password
		passman = urllib2.HTTPPasswordMgrWithDefaultRealm()
		# none is so that all pages under the realm are opened with password
		passman.add_password(None, self.apiUrl, credentials['username'], credentials['password'])

		# authenticate and open the website
		authhandler = urllib2.HTTPBasicAuthHandler(passman)
		opener = urllib2.build_opener(authhandler)
		urllib2.install_opener(opener)

	
		
	